package eu.jonahbauer.raytracing.math;

import org.jetbrains.annotations.NotNull;

/**
 * A vector-like object with three components.
 * @param <T> the type
 */
public interface IVec3<T extends Record & IVec3<T>> extends IVec<T> {
    default double component1() {
        return toVec3().x();
    }
    default double component2() {
        return toVec3().y();
    }
    default double component3() {
        return toVec3().z();
    }

    @Override
    default double get(int i) {
        return switch (i) {
            case 0 -> component1();
            case 1 -> component2();
            case 2 -> component3();
            default -> throw new IndexOutOfBoundsException(i);
        };
    }

    @NotNull Vec3 toVec3();

    @Override
    default double @NotNull [] toArray() {
        return new double[] {component1(), component2(), component3()};
    }
}
