package eu.jonahbauer.raytracing.render.canvas;

import eu.jonahbauer.raytracing.render.spectral.SampledSpectrum;
import eu.jonahbauer.raytracing.render.spectral.SampledWavelengths;
import eu.jonahbauer.raytracing.render.spectral.colors.ColorRGB;
import eu.jonahbauer.raytracing.render.spectral.colors.ColorSpace;
import org.jetbrains.annotations.NotNull;

import javax.swing.*;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;

public final class LiveCanvas implements Canvas {
    private final @NotNull Canvas delegate;
    private final @NotNull BufferedImage image;
    private final @NotNull ColorSpace cs;

    public LiveCanvas(@NotNull Canvas delegate, @NotNull ColorSpace cs) {
        this.delegate = delegate;
        this.image = new BufferedImage(delegate.getWidth(), delegate.getHeight(), BufferedImage.TYPE_INT_RGB);
        this.cs = cs;
    }

    @Override
    public int getWidth() {
        return delegate.getWidth();
    }

    @Override
    public int getHeight() {
        return delegate.getHeight();
    }

    @Override
    public void add(int x, int y, int n, @NotNull SampledSpectrum spectrum, @NotNull SampledWavelengths lambda) {
        delegate.add(x, y, n, spectrum, lambda);
        var color = ColorRGB.gamma(delegate.getRGB(x, y, cs));
        var rgb = color.red() << 16 | color.green() << 8 | color.blue();
        image.setRGB(x, y, rgb);
    }

    @Override
    public @NotNull ColorRGB getRGB(int x, int y, @NotNull ColorSpace cs) {
        return delegate.getRGB(x, y, cs);
    }

    public @NotNull Thread preview() {
        var frame = new JFrame();
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        frame.setContentPane(new JPanel() {
            {
                setPreferredSize(new Dimension(image.getWidth(), image.getHeight()));
            }

            @Override
            protected void paintComponent(Graphics g) {
                g.drawImage(image, 0, 0, null);
            }
        });
        frame.setResizable(false);
        frame.pack();
        frame.setVisible(true);

        var update = Thread.ofVirtual().start(() -> {
            while (!Thread.interrupted()) {
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException ex) {
                    break;
                }
                frame.repaint();
            }
            frame.dispose();
        });

        frame.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                update.interrupt();
            }
        });

        return update;
    }
}
