package eu.jonahbauer.raytracing.render.material;

import eu.jonahbauer.raytracing.math.Ray;
import eu.jonahbauer.raytracing.render.spectral.colors.ColorRGB;
import eu.jonahbauer.raytracing.render.spectral.colors.ColorSpace;
import eu.jonahbauer.raytracing.render.spectral.spectrum.RGBIlluminantSpectrum;
import eu.jonahbauer.raytracing.render.spectral.spectrum.Spectrum;
import eu.jonahbauer.raytracing.render.texture.Texture;
import eu.jonahbauer.raytracing.scene.HitResult;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;
import java.util.Optional;
import java.util.random.RandomGenerator;

public record DiffuseLight(@NotNull Texture texture) implements Material {
    public DiffuseLight {
        Objects.requireNonNull(texture, "texture");
    }

    public DiffuseLight(@NotNull ColorRGB color, @NotNull ColorSpace cs) {
        this(new RGBIlluminantSpectrum(cs, color));
    }

    @Override
    public @NotNull Optional<ScatterResult> scatter(@NotNull Ray ray, @NotNull HitResult hit, @NotNull RandomGenerator random) {
        return Optional.empty();
    }

    @Override
    public @NotNull Spectrum emitted(@NotNull HitResult hit) {
        return texture.get(hit);
    }
}
