package eu.jonahbauer.raytracing.render.spectral.colors;

import eu.jonahbauer.raytracing.render.spectral.spectrum.Spectra;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.io.UncheckedIOException;
import java.util.Objects;

public final class ColorSpaces {
    // Rec. ITU-R BT.709.3
    public static final @NotNull ColorSpace sRGB = new ColorSpace(
            new Chromaticity(0.6400, 0.3300),
            new Chromaticity(0.3000, 0.6000),
            new Chromaticity(0.1500, 0.0600),
            Spectra.D65, read("sRGB_spectrum.bin")
    );
    // P3-D65 (display)
    public static final @NotNull ColorSpace DCI_P3 = new ColorSpace(
            new Chromaticity(0.680, 0.320),
            new Chromaticity(0.265, 0.690),
            new Chromaticity(0.150, 0.060),
            Spectra.D65, read("DCI_P3_spectrum.bin")
    );
    // ITU-R Rec BT.2020
    public static final @NotNull ColorSpace Rec2020 = new ColorSpace(
            new Chromaticity(0.708, 0.292),
            new Chromaticity(0.170, 0.797),
            new Chromaticity(0.131, 0.046),
            Spectra.D65, read("Rec2020_spectrum.bin")
    );

    private static @NotNull SpectrumTable read(@NotNull String name) {
        try (var in = ColorSpaces.class.getResourceAsStream("/eu/jonahbauer/raytracing/colorspace/" + name)) {
            return SpectrumTable.read(Objects.requireNonNull(in));
        } catch (IOException e) {
            throw new UncheckedIOException(e);
        }
    }

    private ColorSpaces() {
        throw new UnsupportedOperationException();
    }
}
