package eu.jonahbauer.raytracing.render.spectral.spectrum;

import eu.jonahbauer.raytracing.render.spectral.colors.ColorSpace;
import eu.jonahbauer.raytracing.render.spectral.colors.SigmoidPolynomial;
import eu.jonahbauer.raytracing.render.spectral.colors.ColorRGB;
import org.jetbrains.annotations.NotNull;

/**
 * A spectrum based on an RGB color used as an illuminant. The spectrum is adjusted to account for the color space's
 * standard illuminant.
 */
public final class RGBIlluminantSpectrum implements Spectrum {
    private final double scale;
    private final @NotNull SigmoidPolynomial polynomial;
    private final @NotNull Spectrum illuminant;

    public RGBIlluminantSpectrum(@NotNull ColorSpace cs, @NotNull ColorRGB rgb) {
        if (rgb.r() < 0 || rgb.g() < 0 || rgb.b() < 0) {
            throw new IllegalArgumentException();
        }
        var max = Math.max(rgb.r(), Math.max(rgb.g(), rgb.b()));
        this.scale = 2 * max;
        this.polynomial = cs.toSpectrum(scale != 0 ? rgb.div(scale) : ColorRGB.BLACK);
        this.illuminant = cs.illuminant();
    }

    @Override
    public double max() {
        return scale * polynomial.max() * illuminant.max();
    }

    @Override
    public double get(double lambda) {
        return scale * polynomial.get(lambda) * illuminant.get(lambda);
    }
}
